#include <stdio.h>
#include <errno.h>
#include <sys/ioctl.h>
#include <fcntl.h>
#include <sys/mman.h>
#include <linux/fb.h>

int main(void)
{
    int x, y, fd = -1;
    struct fb_fix_screeninfo fb_finfo;
    struct fb_var_screeninfo fb_vinfo;
	unsigned int *screen, color, pos, pos_width;
    
    fd = open ("/dev/fb0", O_RDWR);
	if (fd == -1) {
		printf ("can't open %s: %s\n", "/dev/fb0", strerror(errno));
        return -1;
    }
    
	if(ioctl(fd, FBIOGET_FSCREENINFO, &fb_finfo)) {
        printf ("problem with FBITGET_FSCREENINFO ioctl: %s\n", strerror(errno));
        close(fd);
        return -1;
    }

    if (ioctl(fd, FBIOGET_VSCREENINFO, &fb_vinfo)) {
        printf ("problem with FBITGET_VSCREENINFO ioctl: %s\n", strerror(errno));
        close(fd);
        return -1;
    }

    screen = mmap (0, fb_finfo.smem_len, PROT_READ | PROT_WRITE, MAP_SHARED, fd, 0);

    if(screen == (void *)-1) {
        printf ("Couldn't map memory areas: %s\n", strerror(errno));
        close(fd);
        return -1;
    }

	pos_width = fb_vinfo.xres / 8;

	for (y = 0; y < fb_vinfo.yres; y++) {
		color = (y * 3 / fb_vinfo.yres) * 8;
		for (x = 0; x < fb_vinfo.xres; x++) {
			pos = (x / pos_width);
			screen[y * fb_vinfo.xres + x] = (1 << pos) << color;
		}
	}

	munmap(screen, fb_finfo.smem_len);
    close(fd);

	return 0;
}
